// Последнее обновление 2017-05-31 by Phisik
// В рамках разгрузки кода раскидываем функции по файлам
// Тут мы работаем с EEPROM

#include "configuration.h"
#include "declarations.h"

unsigned int ee_addr; // Переменная для хранения текущего адреса чтения/записи в EEPROM
unsigned char eeReadChar()
{
  unsigned char lByte = EEPROM.read(ee_addr);
  ee_addr++;
  return lByte;
}

void eeWriteChar(char p_value)
{
  if (EEPROM.read(ee_addr)!=p_value)
    EEPROM.write(ee_addr, p_value);
  ee_addr++;
}

void eeWriteInt(int p_value)
{
  unsigned char lByte = (unsigned char) (p_value);
  unsigned char hByte =(unsigned char) (p_value >> 8);

  // Для экономии ресурса флеш памяти, сначала проверяем ее содержимое и запись производим только если значение отличается.
  eeWriteChar(lByte);
  eeWriteChar(hByte);
}

//This function will read a 2 byte integer from the eeprom at the specified address and address + 1
unsigned int eeReadInt()
{
  unsigned char lByte = eeReadChar();
  unsigned char hByte = eeReadChar();
  return (unsigned int) (lByte) | (unsigned int)  (hByte << 8);
}

void writeEEPROM()
{ 
  int i; 
  flAutoDetectPowerTEN=0;
  ee_addr=0;
  eeWriteChar(PR_REWRITE_EEPROM);
  eeWriteChar(IspReg);
  eeWriteInt(Power);
  eeWriteInt(UstPowerReg);
  eeWriteInt(TempTerm);
  eeWriteInt(Delta);
  eeWriteChar(FlToUSART);
  eeWriteInt(Temp1P);
  eeWriteInt(Temp2P);
  eeWriteInt(Temp3P);
  for(i=0;i<MAX_DS1820;i++)
    eeWriteChar(ds1820_popr[i]);

  eeWriteInt(tEndRectRazgon);
  eeWriteInt(PowerRect);

  eeWriteInt(tEndRectOtbGlv);
  eeWriteInt(timeChimRectOtbGlv);
  eeWriteChar(ProcChimOtbGlv);
  eeWriteInt(timeChimRectOtbSR);
  eeWriteChar(tDeltaRect);
  eeWriteInt(tEndRectOtbSR);
  eeWriteInt(tEndRect);
  eeWriteInt(PowerDistil);

  eeWriteInt(TempDeflBegDistil);
  eeWriteInt(TempDefl);
  eeWriteChar(DeltaDefl);

  eeWriteChar(BeepEndProcess);
  eeWriteChar(BeepStateProcess);
  eeWriteChar(BeepKeyPress);

  eeWriteInt(tEndDistDefl);
  eeWriteChar(minProcChimOtbSR);
  eeWriteInt(PowerVarkaZerno);
  eeWriteInt(PowerMinute);
  eeWriteInt(NaprPeregrev);
  eeWriteInt(TimeStabKolonna);
  eeWriteChar(DecrementCHIM);
  eeWriteChar(IncrementCHIM);
  eeWriteInt(TimeAutoIncCHIM);
  eeWriteChar(CntCHIM);
  for(i=0;i<COUNT_CHIM;i++)
  {
    eeWriteInt(tempK[i]);
    eeWriteChar(CHIM[i]);
  }
  eeWriteInt(TimeRestabKolonna);
  eeWriteChar(CntPause);
  for(i=0;i<10;i++)
  {
    eeWriteChar(tempP[i]);
    eeWriteChar(timeP[i]);
  }
  eeWriteChar(CorrectASC712);
  eeWriteChar(FlAvtonom);
  eeWriteChar(PIDTemp[0]);
  eeWriteChar(PIDTemp[1]);
  eeWriteChar(PIDTemp[2]);

  eeWriteChar(CountFractionDist);
  for(i=0;i<MAX_CNT_FRACTION_DIST;i++) 
  {
    eeWriteInt(TempFractionDist[i]);
    eeWriteInt(PowerFractionDist[i]);
    eeWriteChar(AngleFractionDist[i]);
  }

}

void readEEPROM()
{
  int i; 
  unsigned char ctmp;
  ee_addr=0;
  eeReadChar();
  IspReg=eeReadChar();
  Power=eeReadInt();
  UstPowerReg=eeReadInt();
  TempTerm=eeReadInt();
  Delta=eeReadInt();
  FlToUSART=eeReadChar();
  Temp1P=eeReadInt();
  Temp2P=eeReadInt();
  Temp3P=eeReadInt();
  for(i=0;i<MAX_DS1820;i++)
    ds1820_popr[i]=eeReadChar();
  tEndRectRazgon=eeReadInt();
  PowerRect=eeReadInt();

  tEndRectOtbGlv=eeReadInt();
  timeChimRectOtbGlv=eeReadInt();
  ProcChimOtbGlv=eeReadChar();
  timeChimRectOtbSR=eeReadInt();
  tDeltaRect=eeReadChar();
  tEndRectOtbSR=eeReadInt();
  tEndRect=eeReadInt();
  PowerDistil=eeReadInt();

  TempDeflBegDistil=eeReadInt();
  TempDefl=eeReadInt();
  DeltaDefl=eeReadChar();

  BeepEndProcess=eeReadChar();
  BeepStateProcess=eeReadChar();
  BeepKeyPress=eeReadChar();

  tEndDistDefl=eeReadInt();
  minProcChimOtbSR=eeReadChar();
  PowerVarkaZerno=eeReadInt();
  PowerMinute=eeReadInt();
  if (PowerMinute>10000 || PowerMinute<0) PowerMinute=Power;
  NaprPeregrev=eeReadInt();
  if (NaprPeregrev>1000 ) NaprPeregrev=300;
  TimeStabKolonna=eeReadInt();
  if (TimeStabKolonna==-1) TimeStabKolonna=900;

  DecrementCHIM=eeReadChar();
  if (DecrementCHIM==-1) DecrementCHIM=10;
  IncrementCHIM=eeReadChar();
  if (IncrementCHIM==-1) IncrementCHIM=0;
  TimeAutoIncCHIM=eeReadInt();
  if (TimeAutoIncCHIM==-1) TimeAutoIncCHIM=600;

  CntCHIM=eeReadChar();
  for(i=0;i<COUNT_CHIM;i++)
  {
    tempK[i]=eeReadInt();
    CHIM[i]=eeReadChar();
  }
  TimeRestabKolonna=eeReadInt();
  if (TimeRestabKolonna==-1) TimeRestabKolonna=3600;

  CntPause=eeReadChar();
  for(i=0;i<10;i++)
  {
    // Если число пауз больше нуля, то считываем их
    if (CntPause>0)
    {
      tempP[i]=eeReadChar();
      timeP[i]=eeReadChar();
    }
    else
    {
      // Иначе просто читаем чтобы счетчик чтений увеличился.
      eeReadChar();
      eeReadChar();
    }
    if (timeP[i]==255) timeP[i]=0;
  }

  if (CntPause==-1) CntPause=8;

  CorrectASC712=eeReadChar();
  if (CorrectASC712==-1) CorrectASC712=0;

  FlAvtonom=eeReadChar();
  if (FlAvtonom==-1) FlAvtonom=0;

  PIDTemp[0]=eeReadChar();
  PIDTemp[1]=eeReadChar();
  PIDTemp[2]=eeReadChar();
  if (PIDTemp[0]==-1)
  {
    PIDTemp[0]=20;
    PIDTemp[1]=10;
    PIDTemp[2]=30;
  }

  CountFractionDist=eeReadChar();
  for(i=0;i<MAX_CNT_FRACTION_DIST;i++) 
  {
    TempFractionDist[i]=eeReadInt();
    if (TempFractionDist[i]==-1)  TempFractionDist[i]=0;
    PowerFractionDist[i]=eeReadInt();
    if (PowerFractionDist[i]==-1)  PowerFractionDist[i]=0;
    AngleFractionDist[i]=eeReadChar();
    if (AngleFractionDist[i]==255)  AngleFractionDist[i]=0;
  }
}
