// Последнее обновление 2018-08-19 by Phisik
// Файл содержит дефолтные настройки системы
// Часть настроек меняется в файле configuration.h при выборе версий, внимательно с этим!


#define PR_REWRITE_EEPROM 0 // Константа, которая содержит признак необходимости перезаписи энергонезависимой памяти (1-254). 
                            // При запуске программы, значение 0-го байта ЕЕПРОМ сравнивается с этим значением, 
                            // и если они не совпадают, тогда энергонезависимая памиять переписывается текущими значениями переменных
                            // То есть для значений переменных из скетча в контроллер, ее значение надо поменять например с 0 на 1.

//=======================================================================================================
// Phisik: настройка UART для работы с периферией

#define DEBUG_SERIAL			 Serial  // Куда выводить отладку
#define DEBUG_SERIAL_BAUDRATE	 115200	

#define MQTT_SERIAL				 Serial3  // Порт, к которому подключена ESP, важно, чтобы не пересекалось с GSM_SERIAL
#define MQTT_SERIAL_BAUDRATE	 250000   // Ставим максимальную, при которой еще нет ошибок связи
#define USE_MQTT_BROKER          1  // MQTT protocol by max506 & limon
                                    // В этом режиме для ESP надо использовать прошивку от limon или мою.

#define MQTT_SERIAL_PING_CHECK   1  // Phisik: Попытка наладить связь между контроллерами при обрыве.
									//         Работает только с моей прошивкой для ESP. С другими прошивками будет только мешать!
									
//=======================================================================================================
// НАСТРОЙКА ЭКРАНА И КНОПОК

// Phisik 2018-07-24: добавил I2C экран. 
// Оригинал здесь http://forum.homedistiller.ru/msg.php?id=13320675
#define LCD_I2C_ADDRESS 0x27		// У каждого экспандера свой адрес. Используйте сканнер I2C.

// Поскольку у людей разные экраны, то лучше мы сделаем тут макрос
#define LCD_WIDTH  16
#define LCD_HEIGHT 2

// предполагаем 2 байта на символ, в международной кодировке (UTF8 может быть 1-4 байта на символ)
#define LCD_BUFFER_SIZE  (LCD_WIDTH*2+2)

//=======================================================================================================
// ОБЩИЕ НАСТРОЙКИ

#define USE_WDT			1		// Поставить 1, если использовать встроенный wath dog
#define BEEP_LONG		15      // Длительность сигнала оповещения о состояниях процесса
#define DEBOUNCE_CYCLES 15      // Число отсчетов таймера для устранения дребезга кнопок, 

#define NO_LOW_POWER_WARNING	 1
#define NO_DETECT_ZERO_WARNING   1  
#define NO_PAGE_BEEP			 1  // Не пищать на 0ой странице
#define USE_ASC712	 0   					 // Если USE_ASC712 = 0, то осуществляется фазовое регулирование. 
                                   // Этот параметр менять не следует!
#define SENSITIVE_ASC712	   205   // Чувствительность датчика тока (показаний АЦП ардуино на 10 ампер тока 135 для 30А датчика, 205 для 20А датчика 82)
 
#define CNT_PERIOD       4			// Количество полу-периодов для обсчета среднеквадратичного

// Update 2018-05-09
// Phisik: этот параметр определяет нужно ли менять температуру стабилизации колонны, если она долго не меняется
// Это связано необходимостью учитывать изменение атмосферного давления. При повышении давления, ситуацию в принципе 
// спасет TimeRestabKolonna, а вот при понижении давления температура уплывает вниз, из-за чего "эффективная" дельта
// становится больше и хвосты можно пропустить. Если у вас есть датчик давления - оно вам скорее всего не надо.
#define ADJUST_COLUMN_STAB_TEMP		0 

#if ADJUST_COLUMN_STAB_TEMP
	// Как часто будем проверять изменение температуры
	const long  tStabCheckPeriod = 10000; // ms
	// С каким весом будем добавлять новую
	const long  tStabTimeConstant = 480; // 360 * 10сек ~ 1 час
	// Множитель для усреднения посчитаем один раз, чтобы не тратить время в loop()
	const float tStabAverageDivisor = 1+1.0/tStabTimeConstant;
#endif

//=======================================================================================================
// НАСТРОЙКА КЛАПАНОВ

// Клапана для управления ШИМ подключены, начиная с PIN 22
// Всего максимум 5 клапанов, то есть на пины с 22 по 26 реализован
// программный ШИМ по количество полупериодов с контролем нуля.

// Phisik:  Не используйте 220В клапана и "ШИМ по количество полупериодов с контролем нуля" - 
//			рано или поздно кого-нибудь прибьет! 
//			Используйте 12ти вольтовые клапана и USE_12V_PWM = 1 для защиты от перегрева

#define KLP_HIGH 1				// Уровень на выходе для сработки клапана
								// Для клапанов с низким уровнем управления поменять 0 на 1
#define PEREGREV_ON 0      // Защита от перегрева клапанов, 1- использовать, 0-нет.
#define USE_12V_PWM 0			// Phisik: Признак того, что надо использовать защиту от перегрева 12В клапанов 

#if PEREGREV_ON==0
    #define PER_KLP_OPEN  1 // клапана открываем через полу-период (аналог диода), чтобы не перегревались и
    #define PER_KLP_CLOSE 1 // чтобы не было гидроударов
#else
    #define PER_KLP_OPEN  1000 // клапана на воду переводим в фазовое управление, чтобы раз в 10 секунд на них подавалось полное напряжение, а затем напряжение
    #define PER_KLP_CLOSE 0    // из контанты U_PEREGREV 150 
#endif

#define MAX_KLP		   4      // Количество клапанов, которыми надо управлять по ШИМ. 

#define KLP_VODA     0      // Номер клапана для управления общей подачей воды в систему. Pin 23
#define KLP_DEFL     1      // Номер клапана для подачи воды в дефлегматор
#define KLP_DEFL_D   1      // Номер клапана для подачи воды в дефлегматор при дистилляции с дефлегматором с паровым отбором
#define KLP_HLD      2      // Номер клапана холодильника для дистилляции
#define KLP_GLV_HVS  2      // Номер клапана отбора головных и хвостовых фракций. Pin 25.
#define KLP_SR       3      // Номер клапана отбора ректификата. Pin 26

//=======================================================================================================
// ДАТЧИКИ ТЕМПЕРАТУРЫ

#define MAX_DS1820		3

// Phisik: update 2018-07-25
// Номера датчиков теперь приведены в порядок. Если датчики определяются не в том порядке, меняем числа тут

#define TEMP_KUB       0    // Номер датчика  термометра в кубе
#define TEMP_RK20      1    // Номер датчика термометра в РК 20 см от насадки
#define TEMP_TSA       2    // Номер термометра в трубке связи с атмосферой
#define TEMP_DEFL TEMP_RK20    // Номер датчика  термометра в дефлегматоре
#define TEMP_TERMOSTAT 0    // Номер датчика  термометра термостата

#define MAX_TEMP_TSA    650	  // Максимальная температура в датчике ТСА
#define MAX_ERR_DS18    120   // После тридцати глюков подряд (а это тридцать секунд) от датчиков температуры, считаем, что процесс надо остановить.

#define MAX_INDEX_INPUT 540
#define MAX_INDEX_BY_PERIOD 90
#define MIN_INDEX_BY_PERIOD 75

//=======================================================================================================
// РЕДАКТИРОВАНИЕ МЕНЮ

// Added by Phisik on 2017-08-15
// Здесь можно отключить ненужные нам пункты в меню

// NB! Пункты 129, 130 всегда должны быть в конце! Пункт 100 - вначале!
//     Это связано с обработчиком нажатий в файле keyboard.cpp, строки 600-630

#define MENU_ITEMS 21 
const bool menuEnableFlag[MENU_ITEMS] = {
  1,    // case 100:  Установка параметров  // Не отключается!
  1,    // case 101:  Displaying   // Отключается, но отключать не стОит
  1,    // case 102:  Термостат    
  1,    // case 103:  Регулятор мощности
  1,    // case 104:  Первый (недробный) отбор
  0,    // case 105:  Отбор голов
  1,    // case 106:  Второй дробный отбор
  1,    // case 107:  Третий дробный отбор 
  0,    // case 108:  Затор зерно
  0,    // case 109:  Ректификация
  1,    // case 110:  Дистилляция с дефлегматором
  0,    // case 111:  НДРФ
  0,    // case 112:  NBK
  0,    // case 113:  Разваривание солодового затора (без варки).
  0,    // case 114:  Разваривание с чиллером и миксером
  1,    // case 115:  Таймер + регулятор мощности
  0,    // case 116:  Пивоварня - клон браумастера
  1,    // case 117:  Фракционная перегонка
  0,    // case 118:  Ректификация Фракционная. // Не Используется.
  1,    // case 129:  Тест клапанов            // Не отключается!   
  0,    // case 130:  Внешнее управление 
};

//=======================================================================================================
// РЕДАКТИРОВАНИЕ МЕНЮ НАСТРОЕК

// Здесь можно отключить ненужные нам пункты в меню настроек
#define SETTINGS_ITEMS 59

// Edited by Phisik on 2017-08-16
// Обновил алгоритм зацикливания. Теперь не надо задавать LAST_ITEM/FIRST_ITEM
// Просто расставить  1/0

const bool settingsEnableFlag[SETTINGS_ITEMS] = {
   1,    //  200: "Max_t_Tst=%5i"
   1,    //  201: "Power TEN=%5u"
   1,    //  202: "Power Reg=%5u"
   0,    //  203: "ParamUSART=%u"
   0,    //  204: "ParamGSM=%u"
   1,    //  205: "dtTermostat=%3i"
   1,    //  206: "Temp 1 Nedrobn Distill=%3i"
   1,    //  207: "Temp 2 Drobn Distill=%3i"
   0,    //  208: "Temp 3 Drobn Distill=%3i"
   0,    //  209: "Temp Razgon Rect (+Kub,-Kol)=%3i"
   0,    //  210: "Power Rectif=%3i"
   1,    //  211: "Vvod Popravok ds18b20 "
   0,    //  212: "Temp Okon Otbor Glv Rectif=%3i"
   0,    //  213: "CHIM Otbor GLV Rectif=%5u"
   0,    //  214: "%% CHIM Otbor GLV Rectif=%3i"
   0,    //  215: "CHIM Otbor SR Rectif=%5u"
   0,    //  216: "Delta Otbor SR Rectif=%3u"
   0,    //  217: "Temp Okon Otbor SR Rectif=%3i"
   0,    //  218: "Temp Okon Rectif=%3i"
   0,    //  219: "Power GLV simple Distill=%4i"
   1,    //  220: "Power simple Distill=%4i"
   1,    //  221: "Temp Begin Dist (+Def -Kub)=%3i"
   1,    //  222: "Temp Distill With Defl=%3i"
   1,    //  223: "Delta Distill With Defl=%3i"
   1,    //  224: "Temp Kub Okon DistWithDefl=%3i"
   0,    //  225: "BeepEndProc=%1u"
   0,    //  226: "BeepStateProc=%1u"
   1,    //  227: "BeepKeyPress=%1u"
   0,    //  228: "PowerRazvarZerno=%4i"
   1,    //  229: "PowerVarkaZerno=%4i"
   0,    //  230: "Period Refresh Server(sec)=%3u"
   1,    //  231: "U Peregrev=%3uV"
   0,    //  232: "Urv Barda=%4i Barda(%4u)"
   0,    //  233: "Provod SR=%4i"
   0,    //  234: "Time Stab (+/-) Kolonna=%5isec"
   0,    //  235: "Edit T & CHIM Count CHIM=%3i"
   0,    //  236: "Auto - CHIM=%3i"
   0,    //  237: "Auto + CHIM=%3i"
   0,    //  238: "Time Auto + CHIM=%5isec"
   0,    //  239: "Time reStab(+/-) Kolonna=%5isec"
   0,    //  240: "Beer Pause Count =%3i"
   1,    //  241: "Power correct ASC712 =%3i"
   0,    //  242: "Server adr= %3u.%3u.%3u.%3u"
   0,    //  243: "Server port= %u"
   0,    //  244: "ID Device= %s"
   0,    //  245: "My Phone= %s"
   0,    //  246: "Alarm Pressure MPX5010=%i"
   0,    //  247: "Use Avtonom HLD=%i"
   0,    //  248: "Time Open BRD=%i"
   0,    //  249: "PID Paramters %4i %4i %4i"
   0,    //  250: "min %% CHIM  Otbor SR=%2i"
   0,    //  251: "Edit Temp Stab Rectif=%3i"
   0,    //  252: "Beg %% CHIM  Otbor SR=%2i"
   0,    //  253: "Popr MPX=%4i %4i/%4i"   
   0,    //  254: "Power NBK=%4i"
   0,    //  255: "Wi-Fi AP= %s"
   0,    //  256: "Wi-Fi Password= %s"
   1,    //  257: "Fraction Dist Count =%3i"
   0,    //  258: "Fraction Rectif Count =%3i"
};

// =======================================================================================================
// ОТКЛЮЧЕНИЕ ЭКРАНОВ

// Здесь можно отключить ненужные нам экраны
#define SCREEN_ITEMS 11

// Edited by Phisik on 2018-07-03
// Просто расставить  1/0

const bool screenEnableFlag[SCREEN_ITEMS] = {
	1, // 1  // Время работы, TicZero, Напр. сети
	1, // 2  // Температуры с датчиков DS18B20
	1, // 3  // Мощность
	0, // 4  // Температура и давление 
	0, // 5  // Сост. аварийных датчиков. 
	1, // 6  // Регулятор мощности
	0, // 7  // Состояние упр. клапанами
	1, // 8  // Текущий процесс
	0, // 9  // Состояние ПИД
	0, // 10 // Макс. темп. и давление
	0  // 11 // Расчёт ПИД
};
